"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var touch_gesture_listener_1 = require("./lib/touch-gesture-listener");
var dom_1 = require("./lib/dom");
var listen_1 = require("./lib/listen");
var SwipeAwaySheet = /** @class */ (function () {
    function SwipeAwaySheet(sheet, options) {
        var _this = this;
        this.sheet = sheet;
        this.options = options;
        this.translated = 0;
        var backdrop = (this.backdrop = dom_1.el("div", {
            className: "sheet-backdrop"
        }));
        this.sheetContent = sheet.querySelector(".sheet-content");
        this.container = dom_1.el("div", { className: "sheet-container" });
        this.container.appendChild(backdrop);
        this.sheet.parentElement.insertBefore(this.container, this.sheet);
        this.container.appendChild(sheet);
        this.backdropGestureListener = touch_gesture_listener_1.touchGestureListener(backdrop, function (_a) {
            var tap = _a.tap;
            tap(function () {
                _this.close();
            });
        });
        this.touchStartListener = touch_gesture_listener_1.touchGestureListener(sheet, function (_a) {
            var startEv = _a.ev, move = _a.move, end = _a.end, cancel = _a.cancel;
            var startTouch = {
                time: Date.now(),
                y: touch_gesture_listener_1.getEventY(startEv),
                x: touch_gesture_listener_1.getEventX(startEv)
            };
            var lastTouch = { time: Date.now(), y: touch_gesture_listener_1.getEventY(startEv) };
            var secondLastTouch = null;
            var axis = null;
            move(function (moveEv) {
                var thisTouch = {
                    time: Date.now(),
                    y: touch_gesture_listener_1.getEventY(moveEv),
                    x: touch_gesture_listener_1.getEventX(moveEv)
                };
                var direction = thisTouch.y > startTouch.y ? "down" : "up";
                axis =
                    axis ||
                        Math.abs(thisTouch.y - startTouch.y) >
                            Math.abs(thisTouch.x - startTouch.x)
                        ? "vertical"
                        : "horizontal";
                var shouldAllowScroll = Array.from(dom_1.itemAndParents(startEv.target))
                    .filter(function (el) { return el === sheet || dom_1.isDescendantOf(el, sheet); })
                    .some(function (el) {
                    if (el === document.body || el === document.documentElement) {
                        return false;
                    }
                    var style = window.getComputedStyle(el);
                    var overflowY = style.getPropertyValue("overflow-y");
                    var isScrollable = overflowY === "auto" || overflowY === "scroll";
                    var canScroll = el.scrollHeight > el.offsetHeight;
                    if (!canScroll || !isScrollable) {
                        return false;
                    }
                    var height = el.clientHeight;
                    var isAtBottom = el.scrollHeight - el.scrollTop <= height;
                    var isAtTop = el.scrollTop <= 0;
                    if (direction === "up" && !isAtBottom) {
                        return true;
                    }
                    if (direction === "down" && !isAtTop) {
                        return true;
                    }
                    return false;
                });
                if ((shouldAllowScroll && _this.translated === 0) ||
                    axis === "horizontal") {
                    cancel();
                    return;
                }
                moveEv.preventDefault();
                var translated = _this.translated + thisTouch.y - lastTouch.y;
                sheet.style.transition = "none";
                backdrop.style.transition = "none";
                _this.translateSheet(translated);
                secondLastTouch = lastTouch;
                lastTouch = thisTouch;
            });
            end(function (endEv) {
                if (secondLastTouch == null) {
                    return;
                }
                var thisTouch = { time: Date.now() };
                var velocity = (lastTouch.y - secondLastTouch.y) /
                    (thisTouch.time - secondLastTouch.time);
                var selectedStop = _this.selectAStop(_this.sheet.clientHeight - _this.translated, velocity);
                if (selectedStop != null) {
                    _this.transitionTo(selectedStop);
                }
                else {
                    _this.close();
                }
            });
        });
    }
    SwipeAwaySheet.prototype.selectAStop = function (y, velocity) {
        if (Math.abs(velocity) < 0.15) {
            velocity = 0;
        }
        var stops = [this.sheet.clientHeight].concat((this.options.stops || []));
        var stopsByDistanceFromY = stops.sort(function (a, b) { return Math.abs(a - y) - Math.abs(b - y); });
        var direction = velocity === 0 ? "none" : velocity > 0 ? "down" : "up";
        var candidateStops = stopsByDistanceFromY.filter(function (stop) {
            return direction === "none" ? true : direction === "up" ? stop > y : stop < y;
        });
        if (candidateStops.length === 0 && direction === "up") {
            return this.sheet.clientHeight;
        }
        return candidateStops[0];
    };
    SwipeAwaySheet.prototype.translateSheet = function (y) {
        if (y < 0) {
            // trying to drag higher than it can go, should resist
            this.sheet.style.transform = "translate(-50%,0px)";
            this.sheet.style.paddingBottom = -y / 2 + "px";
        }
        else {
            // this is fine
            this.sheet.style.transform = "translate(-50%," + y + "px)";
            this.sheet.style.paddingBottom = "0px";
        }
        this.translated = y;
        this.backdrop.style.opacity = String(1 - this.translated / this.sheet.clientHeight);
    };
    SwipeAwaySheet.prototype.open = function () {
        this.sheetContent.scrollTo(0, 0);
        var stop = (this.options.stops && this.options.stops[0]) || 0;
        this.transitionTo(stop);
    };
    SwipeAwaySheet.prototype.transitionTo = function (overhangHeight) {
        if (overhangHeight === 0) {
            return this.close();
        }
        var translateHeight = Math.max(0, this.sheet.clientHeight - overhangHeight);
        this.sheet.style.transition = "350ms cubic-bezier(0.29, 0.1, 0.44, 1) all";
        this.backdrop.style.transition = "350ms cubic-bezier(0.175, 0.885, 0.32, .975) opacity";
        this.translateSheet(translateHeight);
        this.container.style.pointerEvents = "auto";
    };
    SwipeAwaySheet.prototype.setValue = function (value) {
        this.value = value;
    };
    SwipeAwaySheet.prototype.close = function (time, value) {
        var _this = this;
        if (time === void 0) { time = 350; }
        time = Math.min(time, 350);
        this.sheet.style.transition = time *
            0.6 + "ms cubic-bezier(0.35, 0.15, 0.85, .6) all";
        this.backdrop.style.transition = time *
            0.6 + "ms cubic-bezier(0.35, 0.15, 0.85, .6) opacity";
        this.translateSheet(this.sheet.clientHeight);
        this.container.style.pointerEvents = "none";
        listen_1.listen(this.sheet, [transitionEndEvent], function () {
            _this.options.onClose(value === undefined ? _this.value : value);
        });
    };
    SwipeAwaySheet.prototype.destroy = function () {
        this.touchStartListener();
        this.backdropGestureListener();
    };
    return SwipeAwaySheet;
}());
exports.SwipeAwaySheet = SwipeAwaySheet;
var ɵ0 = function () {
    var transitions = {
        transition: "transitionend",
        WebkitTransition: "webkitTransitionEnd",
        MozTransition: "transitionend",
        OTransition: "otransitionend"
    }, elem = document.createElement("div");
    for (var t in transitions) {
        if (typeof elem.style[t] !== "undefined") {
            return transitions[t];
        }
    }
};
exports.ɵ0 = ɵ0;
var transitionEndEvent = (ɵ0)();
//# sourceMappingURL=sheet.js.map